@@ -56,13 +56,13 @@
 typedef struct
 {
    /* Sign (+1, -1) or 0 for the value zero */
-   int sign; //, im_sign;
+   int sign, im_sign;
 
    /* Exponent (to base MP_BASE) */
-   int exponent; //, im_exponent;
+   int exponent, im_exponent;
 
    /* Normalized fraction */
-   int fraction[MP_SIZE]; //, im_fraction[MP_SIZE];
+   int fraction[MP_SIZE], im_fraction[MP_SIZE];
 } MPNumber;
 
 typedef enum
@@ -167,11 +167,20 @@ void   mp_divide_integer(const MPNumber *x, int64_t y, MPNumber *z);
 /* Sets z = 1 ÷ x */
 void   mp_reciprocal(const MPNumber *, MPNumber *);
 
-/* Sets z = fractional part of x */
+/* Sets z = x mod 1 */
 void   mp_fractional_component(const MPNumber *x, MPNumber *z);
 
-/* Sets z = integer part of x */
-void   mp_integer_component(const MPNumber *x, MPNumber *z);
+/* Sets z = {x} */
+void   mp_fractional_part(const MPNumber *x, MPNumber *z);
+
+/* Sets z = ⌊x⌋ */
+void   mp_floor(const MPNumber *x, MPNumber *z);
+
+/* Sets z = ⌈x⌉ */
+void   mp_ceiling(const MPNumber *x, MPNumber *z);
+
+/* Sets z = [x] */
+void   mp_round(const MPNumber *x, MPNumber *z);
 
 /* Sets z = ln x */
 void   mp_ln(const MPNumber *x, MPNumber *z);
