@@ -25,7 +25,6 @@
 #include "mp-equation.h"
 
 static MPEquationOptions options;
-static int base;
 
 static int fails = 0;
 
@@ -84,7 +83,7 @@ test(char *expression, char *expected, int expected_error)
     error = mp_equation_parse(expression, &options, &result, NULL);
 
     if(error == 0) {
-        mp_cast_to_string(&result, base, 9, 1, result_str, 1024);
+        mp_cast_to_string(&result, options.base, options.base, 9, 1, result_str, 1024);
         if(expected_error != 0)
             fail("'%s' -> %s, expected error %s", expression, result_str, error_code_to_string(expected_error));
         else if(strcmp(result_str, expected) != 0)
@@ -106,7 +105,7 @@ static void
 test_conversions()
 {
     memset(&options, 0, sizeof(options));
-    base = 10;
+    options.base = 10;
     options.wordlen = 32;
     options.angle_units = MP_DEGREES;
 
@@ -130,7 +129,7 @@ test_conversions()
 
 
 int
-variable_is_defined(const char *name)
+variable_is_defined(const char *name, void *data)
 {
     return strcmp (name, "x") == 0 || strcmp (name, "y") == 0;
 }
@@ -160,23 +159,24 @@ void
 test_equations()
 {
     memset(&options, 0, sizeof(options));
-    base = 10;
+    options.base = 10;
     options.wordlen = 32;
     options.angle_units = MP_DEGREES;
     options.variable_is_defined = variable_is_defined;  
     options.get_variable = get_variable;
     options.set_variable = set_variable;
 
-    base = 2;
-    test("2", "10₂", 0);
+    options.base = 2;
+    test("2₁₀", "10", 0);
 
-    base = 8;
-    test("16434824", "76543210₈", 0);
+    options.base = 8;
+    test("16434824₁₀", "76543210", 0);
 
-    base = 16;
-    test("18364758544493064720", "FEDCBA9876543210₁₆", 0);
+    options.base = 16;
+    test("FF", "FF", 0);
+    test("18364758544493064720₁₀", "FEDCBA9876543210", 0);
 
-    base = 10;
+    options.base = 10;
     test("0₂", "0", 0); test("0₈", "0", 0); test("0", "0", 0); test("0₁₆", "0", 0);
     test("1₂", "1", 0); test("1₈", "1", 0); test("1", "1", 0); test("1₁₆", "1", 0);
     test("2₂", "", PARSER_ERR_INVALID); test("2₈", "2", 0); test("2", "2", 0); test("2₁₆", "2", 0);
@@ -210,6 +210,12 @@ test_equations()
     test("256", "256", 0);
     test("½", "0.5", 0);
     test("1½", "1.5", 0);
+    test("0°", "0", 0);
+    test("1°", "1", 0);
+    test("0°30'", "0.5", 0);
+    //test("0°0.1'", "1", 0); // FIXME: Not yet supported
+    test("0°0'1\"", "0.000277778", 0);
+    test("0°0'0.1\"", "0.000027778", 0);
     test("1.00", "1", 0);
     test("1.01", "1.01", 0);
 
@@ -372,7 +378,6 @@ test_equations()
     test("4^3^2", "262144", 0);
     test("4^(3^2)", "262144", 0);
     test("(4^3)^2", "4096", 0);
-    //test("e^(iπ)", "1", 0);
     test("√4", "2", 0);
     test("√4−2", "0", 0);
     test("∛8", "2", 0);
@@ -386,7 +391,7 @@ test_equations()
     test("Sqrt(2)", "1.414213562", 0);
     test("4^0.5", "2", 0);
     test("2^0.5", "1.414213562", 0);
-    test("(−4)^0.5", "", PARSER_ERR_MP);
+    test("₃√−8", "−2", 0);
     test("(−8)^(1÷3)", "−2", 0);
 
     test("0 mod 7", "0", 0);
@@ -395,10 +400,32 @@ test_equations()
     test("8 mod 7", "1", 0);
     test("−1 mod 7", "6", 0);
 
-    test("int 3.2", "3", 0);
-    test("frac 3.2", "0.2", 0);
-    test("int (−3.2)", "−3", 0);
-    test("frac (−3.2)", "−0.2", 0);
+    test("⌊3⌋", "3", 0);
+    test("⌈3⌉", "3", 0);
+    test("[3]", "3", 0);
+    test("⌊−3⌋", "−3", 0);
+    test("⌈−3⌉", "−3", 0);
+    test("[−3]", "−3", 0);
+    test("⌊3.2⌋", "3", 0);
+    test("⌈3.2⌉", "4", 0);
+    test("[3.2]", "3", 0);
+    test("⌊−3.2⌋", "−4", 0);
+    test("⌈−3.2⌉", "−3", 0);
+    test("[−3.2]", "−3", 0);
+    test("⌊3.5⌋", "3", 0);
+    test("⌈3.5⌉", "4", 0);
+    test("[3.5]", "4", 0);
+    test("⌊−3.5⌋", "−4", 0);
+    test("⌈−3.5⌉", "−3", 0);
+    test("[−3.5]", "−4", 0);
+    test("⌊3.7⌋", "3", 0);
+    test("⌈3.7⌉", "4", 0);
+    test("[3.7]", "4", 0);
+    test("⌊−3.7⌋", "−4", 0);
+    test("⌈−3.7⌉", "−3", 0);
+    test("[−3.7]", "−4", 0);
+    test("{3.2}", "0.2", 0);
+    test("{−3.2}", "0.8", 0);
 
     test("|1|", "1", 0);
     test("|−1|", "1", 0);
@@ -407,7 +434,6 @@ test_equations()
     test("abs 1", "1", 0);
     test("abs (−1)", "1", 0);
 
-    test("log (−1)", "", PARSER_ERR_MP);
     test("log 0", "", PARSER_ERR_MP);
     test("log 1", "0", 0);
     test("log 2", "0.301029996", 0);
@@ -416,7 +442,6 @@ test_equations()
     test("log₂ 2", "1", 0);
     test("2 log 2", "0.602059991", 0);
 
-    test("ln (−1)", "", PARSER_ERR_MP);
     test("ln 0", "", PARSER_ERR_MP);
     test("ln 1", "0", 0);
     test("ln 2", "0.693147181", 0);
@@ -479,6 +504,38 @@ test_equations()
     options.angle_units = MP_GRADIANS;
     test("sin 100", "1", 0);
 
+    /* Complex numbers */
+    options.angle_units = MP_DEGREES;
+    test("i", "i", 0);
+    test("−i", "−i", 0);
+    test("2i", "2i", 0);
+    test("1+i", "1+i", 0);  
+    test("i+1", "1+i", 0);
+    test("1−i", "1−i", 0);  
+    test("i−1", "−1+i", 0);
+    test("i×i", "−1", 0);
+    test("i÷i", "1", 0);
+    test("1÷i", "−i", 0);
+    test("|i|", "1", 0);
+    test("|3+4i|", "5", 0);
+    test("arg 0", "", PARSER_ERR_MP);  
+    test("arg 1", "0", 0);
+    test("arg (1+i)", "45", 0);
+    test("arg i", "90", 0);
+    test("arg (−1+i)", "135", 0);
+    test("arg −1", "180", 0);
+    test("arg (1+−i)", "−45", 0);
+    test("arg −i", "−90", 0);
+    test("arg (−1−i)", "−135", 0);
+    test("i⁻¹", "−i", 0);
+    test("√−1", "i", 0);
+    test("(−1)^0.5", "i", 0);
+    test("√−4", "2i", 0);
+    test("e^iπ", "−1", 0);
+    test("log (−10) − (1 + πi÷ln(10))", "0", 0);
+    test("ln (−e) − (1 + πi)", "0", 0);
+
+    /* Boolean */
     test("0 and 0", "0", 0);
     test("1 and 0", "0", 0);
     test("0 and 1", "0", 0);
@@ -497,12 +554,24 @@ test_equations()
     test("1 xor 1", "0", 0);
     test("3 xor 5", "6", 0);
 
-    base = 16;
-    test("ones 1", "FFFFFFFE₁₆", 0);
-    test("ones 7FFFFFFF₁₆", "80000000₁₆", 0);
-    test("twos 1", "FFFFFFFF₁₆", 0);
-    test("twos 7FFFFFFF₁₆", "80000001₁₆", 0);
-    test("~7A₁₆", "FFFFFF85₁₆", 0);
+    options.base = 16;
+    test("ones 1", "FFFFFFFE", 0);
+    test("ones 7FFFFFFF", "80000000", 0);
+    test("twos 1", "FFFFFFFF", 0);
+    test("twos 7FFFFFFF", "80000001", 0);
+    test("~7A₁₆", "FFFFFF85", 0);
+
+    options.base = 2;
+    options.wordlen = 4;
+    test("1100∧1010", "1000", 0);
+    test("1100∨1010", "1110", 0);
+    test("1100⊻1010", "110", 0);
+    test("1100⊕1010", "110", 0);
+    //test("1100⊼1010", "0111", 0);
+    //test("1100⊽1010", "0001", 0);
+    //options.wordlen = 2;
+    //test("¬01₂", "10₂", 0);
+    //test("¬¬10₂", "10₂", 0);
 }
 
 
@@ -527,7 +596,7 @@ test_string(const char *number)
 {
     MPNumber t;
 
-    mp_set_from_string(number, &t);
+    mp_set_from_string(number, 10, &t);
 
     printf("MPNumber(%s) -> {", number);
     print_number(&t);
@@ -678,7 +747,7 @@ test_mp()
 
 void
 unittest()
-{
+{    
     test_mp();
     test_numbers();
     test_conversions();
